var mat4 = glMatrix.mat4;
var vec3 = glMatrix.vec3;

var state = {
	drag : {
		dragging : false,
		startX : -1,
		startY : -1,
	},
	camera : {
		dirty : true,
		angleX : 90,
		angleY : 0,
		distance : 80,
		viewMatrix : mat4.create(),
		projectionMatrix : mat4.create(),
		mvpPosition : null,
	},
	nrOfVertices: 3,
	
}
var vertexBuffer;
var gl;
var loop;
var mv;
var mvp;
var eyeVec;
var sx;
var cx;
var sy;
var cy;
var stop = false;
var resizeCanvas;

var vertexShaderText = 
[
'precision mediump float;',
'',
'attribute vec3 vertPosition;',
'attribute vec3 vertColor;',
'attribute vec3 vertNormal;',
'',
'uniform mat4 mvp;',
'',
'varying vec3 fragColor;',
'',
'void main()',
'{',
'',
'  float diff = dot(vertNormal, vec3(0.99602, 0.04193, 0.0786));',
'  diff = diff * 0.3 + 0.7;',
'  vec3 diffuse = diff * vertColor;',
'  fragColor = diffuse;',
'  gl_Position = mvp * vec4(vertPosition, 1.0);',
'}'
].join('\n');

var fragmentShaderText =
[
'precision mediump float;',
'',
'varying vec3 fragColor;',
'',
'void main()',
'{',
'  gl_FragColor = vec4(fragColor, 1.0);',
'}'
].join('\n');


$(document).ready(function() {
	var canvas = document.getElementById('testCanvas');
	
	resizeCanvas = function () {
		var width = canvas.clientWidth;
		var height = canvas.clientHeight;
		if (canvas.width != width || canvas.height != height) {
			canvas.width = width;
			canvas.height = height;
			mat4.perspective(state.camera.projectionMatrix, glMatrix.glMatrix.toRadian(45),
					canvas.width / canvas.height, 0.1, 1000.0);
			state.camera.dirty = true;
		}
	}

	window.addEventListener('resize', resizeCanvas);
	
	setupMouseListeners(canvas);
	
	gl = canvas.getContext('webgl');
	
	if (!gl) {
		console.log('WebGL not supported, switching to experimental web gl');
		gl = canvas.getContext('experimental-webgl');
	}
	
	if (!gl) {
		alert('Browser does not support WebGL');
		return;
	}
		
	gl.clearColor(0.75, 0.85, 0.8, 1.0);
	gl.enable(gl.DEPTH_TEST)
	
	var program = setupShaders(gl);
	if (!program) {
		return;
	}
	gl.useProgram(program);
	eyeVec = vec3.create();
	state.camera.mvpPosition = gl.getUniformLocation(program, 'mvp');
	mat4.perspective(state.camera.projectionMatrix, glMatrix.glMatrix.toRadian(45),
			canvas.width / canvas.height, 0.1, 1000.0);
	
	resetCamera();
	mv = mat4.create();
	mvp = mat4.create();
	
	vertexBuffer = gl.createBuffer();
	gl.bindBuffer(gl.ARRAY_BUFFER, vertexBuffer);
	
	var attribLoc = gl.getAttribLocation(program, 'vertPosition');
	gl.vertexAttribPointer(attribLoc, 3, gl.FLOAT, false, 9 * Float32Array.BYTES_PER_ELEMENT, 0);
	gl.enableVertexAttribArray(attribLoc);
	
	var colorAttrib = gl.getAttribLocation(program, 'vertColor');
	gl.vertexAttribPointer(colorAttrib, 3, gl.FLOAT, false, 9 * Float32Array.BYTES_PER_ELEMENT,
			6 * Float32Array.BYTES_PER_ELEMENT);
	gl.enableVertexAttribArray(colorAttrib);
	
	var normalAttrib = gl.getAttribLocation(program, 'vertNormal');
	gl.vertexAttribPointer(normalAttrib, 3, gl.FLOAT, false, 9 * Float32Array.BYTES_PER_ELEMENT,
			3 * Float32Array.BYTES_PER_ELEMENT);
	gl.enableVertexAttribArray(normalAttrib);
	
	resizeCanvas();
	gl.clear(gl.COLOR_BUFFER_BIT | gl.DEPTH_BUFFER_BIT);
	loop = function () {
		resizeCanvas();
		gl.viewport(0, 0, gl.canvas.width, gl.canvas.height);
		gl.clear(gl.COLOR_BUFFER_BIT | gl.DEPTH_BUFFER_BIT);
		if (state.camera.dirty) {
			state.camera.dirty = false;
			updateCamera();
		}
		gl.drawArrays(gl.TRIANGLES, 0, state.nrOfVertices);

		if (!stop) {
			requestAnimationFrame(loop);
		} else {
			gl.clear(gl.COLOR_BUFFER_BIT | gl.DEPTH_BUFFER_BIT);
		}
	}
});

function updateCamera() {
	sx = Math.sin(glMatrix.glMatrix.toRadian(state.camera.angleX));
	cx = Math.cos(glMatrix.glMatrix.toRadian(state.camera.angleX));
	sy = Math.sin(glMatrix.glMatrix.toRadian(state.camera.angleY));
	cy = Math.cos(glMatrix.glMatrix.toRadian(state.camera.angleY));
	eyeVec[0] = state.camera.distance * sx * cy;
	eyeVec[1] = state.camera.distance * sx * sy;
	eyeVec[2] = state.camera.distance * cx;
	
	mat4.lookAt(state.camera.viewMatrix, eyeVec, [0, 0, 0], [0, 0, 1]);
	
	mat4.multiply(mvp, state.camera.projectionMatrix, state.camera.viewMatrix);
	gl.uniformMatrix4fv(state.camera.mvpPosition, false, mvp);
}

function resetCamera() {
	state.camera.angleX = 60;
	state.camera.angleY = 330;
	state.camera.dirty = true;
	mat4.lookAt(state.camera.viewMatrix, [state.camera.distance, 0, 0], [0, 0, 0], [0, 0, 1]);
}

function drawModel(nrOfVertices, byteArray, width) {
	stop = false;
	state.camera.distance = Math.max(25, 1.5 * width);
	state.camera.dirty = true;
	state.nrOfVertices = nrOfVertices;
	gl.bindBuffer(gl.ARRAY_BUFFER, vertexBuffer);
	gl.bufferData(gl.ARRAY_BUFFER, byteArray, gl.STATIC_DRAW);
	requestAnimationFrame(loop);
}

function stopRender() {
	stop = true;
}

function setupShaders(gl) {
	var vertexShader = gl.createShader(gl.VERTEX_SHADER);
	var fragmentShader = gl.createShader(gl.FRAGMENT_SHADER);
	
	gl.shaderSource(vertexShader, vertexShaderText);
	gl.shaderSource(fragmentShader, fragmentShaderText);
	
	gl.compileShader(vertexShader);
	if (!gl.getShaderParameter(vertexShader, gl.COMPILE_STATUS)) {
		console.error('ERROR compiling vertex shader!', gl.getShaderInfoLog(vertexShader));
		return 0;
	}
	
	gl.compileShader(fragmentShader);
	if (!gl.getShaderParameter(fragmentShader, gl.COMPILE_STATUS)) {
		console.error('ERROR compiling fragment shader!', gl.getShaderInfoLog(fragmentShader));
		return 0;
	}
	
	var program = gl.createProgram();
	gl.attachShader(program, vertexShader);
	gl.attachShader(program, fragmentShader);
	gl.linkProgram(program);
	if (!gl.getProgramParameter(program, gl.LINK_STATUS)) {
		console.error('ERROR linking program!', gl.getProgramInfoLog(program));
		return 0;
	}
	
	gl.validateProgram(program);
	if (!gl.getProgramParameter(program, gl.VALIDATE_STATUS)) {
		console.error('ERROR validating program!', gl.getProgramInfoLog(program));
		return 0;
	}
	return program;
}

function zoom(event) {
	 event.preventDefault();
	 state.camera.distance += event.deltaY * 0.8;
	 state.camera.dirty = true;
}

function setupMouseListeners(canvas) {
	canvas.onwheel = zoom;
	
	canvas.onmousedown = function (e) {
		// left mouse button
		if (e.buttons == 1 && e.button == 0) {
			state.drag.dragging = true;
			state.drag.startX = e.x;
			state.drag.startY = e.y;
		}
	}
	
	canvas.onmousemove = function (e) {
		if (state.drag.dragging && e.buttons == 1 && e.button == 0) {
			var difX = (e.x - state.drag.startX) / 2.0;
			var difY = (e.y - state.drag.startY) / 2.0;
			state.drag.startX = e.x;
			state.drag.startY = e.y;
			state.camera.angleX -= difY;
			state.camera.angleY -= difX;
			if (state.camera.angleX < 1) {
				state.camera.angleX = 1;
			} else if (state.camera.angleX >= 179) {
				state.camera.angleX = 179;
			}
			if (state.camera.angleY < 0) {
				state.camera.angleY += 360;
			} else if (state.camera.angleY >= 360) {
				state.camera.angleY -= 360;
			}
			state.camera.dirty = true;
		}
	}
	
	canvas.onmouseup = function (e) {
		state.drag.dragging = false;
	}
}